library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity Multiplier is
	port(A, B : in std_logic_vector(15 downto 0);
		O : out std_logic_vector(31 downto 0));
end Multiplier;

architecture Behavioral of Multiplier is

component MultiplierCore
	port(A, B, I : in std_logic;
		Cin : in std_logic;
		Cout : out std_logic;
		O : out std_logic);
end component;

component Adder
	generic(W : integer := 8);
	port(Sigma : in std_logic;
		A, B : in std_logic_vector(W - 1 downto 0);
		O : out std_logic_vector(W - 1 downto 0);
		Cout : out std_logic);
end component;

	type signal_array is array (32 downto 0) of std_logic_vector(32 downto 0);
	signal IO : signal_array;
	signal Carry : signal_array;
	
	signal AdderCarryOut : std_logic;
	signal AA, AB, AO : std_logic_vector(14 downto 0);
	
begin
	
	U1 : for i in 0 to 15 generate
		U2 : for j in i to i + 15 generate
			U3 : MultiplierCore
				port map(A(i), B(j - i), 
						IO(i)(j),
						Carry(i)(j), Carry(i + 1)(j + 1),
						IO(i + 1)(j));
		end generate;
	end generate;
	
	U4 : for j in 0 to 31 generate
		Carry(0)(j) <= '0';
		IO(0)(j) <= '0';
	end generate;
	
	U5 : for i in 0 to 15 generate
		IO(i + 1)(i + 16) <= '0';
		O(i) <= IO(i + 1)(i);
	end generate;
	
	U6 : Adder
		generic map(15)
		port map('0', AA, AB, AO, AdderCarryOut);
	
	AA <= Carry(16)(30 downto 16);
	AB <= IO(16)(30 downto 16);
	
	O(30 downto 16) <= AO;
	
	O(31) <= AdderCarryOut xor Carry(16)(31);
	
end Behavioral;
